﻿using System;
using System.Collections;
using System.Collections.Generic;

using System.Data;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using Dapfor.Wpf.Converters;
using Dapfor.Wpf.Grid.Demo.Internal;


namespace Dapfor.Wpf.Grid.Demo.Examples.Basics.FilteringAndGrouping
{
    /// <summary>
    /// Interaction logic for FilteringAndGroupingDemo.xaml
    /// </summary>
    [Example("Basics", "Simultanous filtering and grouping", "Description_FilteringAndGroupingDemo")]
    public partial class FilteringAndGroupingDemo : IDisposable
    {
        private static readonly DataTable OrdersTable;

        static FilteringAndGroupingDemo()
        {
            //Create some products
            IList<string> products = new List<string>();
            for (int i = 1; i < 10; ++i)
            {
                products.Add(string.Format("Product {0}", i));
            }

            OrdersTable = new DataTable("Orders");
            OrdersTable.Columns.Add("orderId", typeof(int)).AutoIncrement = true;
            OrdersTable.Columns.Add("product", typeof(string));
            OrdersTable.Columns.Add("amount", typeof(double));
            OrdersTable.Columns.Add("quantity", typeof(long));
            OrdersTable.PrimaryKey = new[] { OrdersTable.Columns["orderId"] };

            //populate the table
            var random = new Random();
            for (int orderId = 1; orderId <= 1000; orderId++)
            {
                // add customer record
                OrdersTable.Rows.Add(new object[] { orderId, products[random.Next() % products.Count], (double)random.Next(20, 100) / random.Next(1, 10), (long)random.Next(1, 10) });
            }
        }

        public FilteringAndGroupingDemo()
        {
            InitializeComponent();

            //grid.DataMember = "Orders";
            grid.ItemsSource = OrdersTable;
            tbRowCount.Text = string.Format("{0}", OrdersTable.Rows.Count);

            grid.Headers[0]["amount"].ValueConverter = new DoubleConverter(2);
        }

        private void OnUpdateGridFilter(object sender, TextChangedEventArgs e)
        {
            var tb = sender as TextBox;
            var expression = tb != null ? tb.GetBindingExpression(TextBox.TextProperty) : null;
            if(expression != null)
            {
                expression.UpdateSource();
            }
            if (grid != null)
            {
                grid.FilterRefresh();    
            }
        }

        private void OnCheckChanged(object sender, System.Windows.RoutedEventArgs e)
        {
            //Update the filter when text is changed
            var cb = sender as CheckBox;
            var expression = cb != null ? cb.GetBindingExpression(ToggleButton.IsCheckedProperty) : null;
            if (expression != null)
            {
                expression.UpdateSource();
            }

            //Refilter rows in grid
            if (grid != null)
            {
                grid.FilterRefresh();
            }
        }

        private void grid_RowVisibilityChanged(object sender, Controls.GridControlRowEventArgs e)
        {
            tbVisibleRowCount.Text = string.Format("{0}", grid.Rows.Count);
        }

        public void Dispose()
        {
            grid.ItemsSource = null;
        }
    }
}