﻿using System;
using System.ComponentModel;
using Dapfor.Wpf.Converters;
using Dapfor.Wpf.Data;

namespace Dapfor.Wpf.Grid.Demo.Examples.DataBinding.ConditionalHierarchicalBinding
{
    public class Order : INotifyPropertyChanged
    {
        #region Fields

        private double _price;
        private long _quantity;
        private Status _status;
        private Way _way;

        #endregion

        public Order(string id, Way way, double price, long quantity, string product, Basket basket)
        {
            Deals = new BindingList<Deal>();
            Id = id;
            Product = product;
            _way = way;
            _price = price;
            _quantity = quantity;
            Basket = basket;
            CreationDate = LastChangeTimeStamp = DateTime.Now;
        }

        #region Public properties

        public BindingList<Deal> Deals { get; private set; }

        public string Id { get; private set; }

        public string Product { get; private set; }

        public Basket Basket { get; private set; }

        [Field("TimeStamp")]
        [Converter("HH:mm:ss.fff")]
        public DateTime CreationDate { get; private set; }

        public DateTime LastChangeTimeStamp { get; private set; }

        public Way Way
        {
            get { return _way; }
            set
            {
                UpdateChangeTimeStamp();
                if (!Equals(_way, value))
                {
                    _way = value;
                    FirePropertyChanged("Way");
                }
            }
        }

        [DoubleConverter(3)]
        public double Price
        {
            get { return _price; }
            set
            {
                UpdateChangeTimeStamp();
                if (_price != value)
                {
                    _price = value;
                    FirePropertyChanged("Price");
                }
            }
        }

        public long Quantity
        {
            get { return _quantity; }
            set
            {
                UpdateChangeTimeStamp();
                if (_quantity != value)
                {
                    _quantity = value;
                    FirePropertyChanged("Quantity");
                }
            }
        }


        public Status Status
        {
            get { return _status; }
            set
            {
                if (!Equals(_status, value))
                {
                    _status = value;
                    FirePropertyChanged("Status");
                }
            }
        }

        #endregion

        #region Public methods

        public void AddDeal(Deal deal)
        {
            if (deal == null)
            {
                throw new ArgumentNullException("deal");
            }
            UpdateChangeTimeStamp();
            Deals.Add(deal);
        }

        #endregion

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        #endregion

        #region Private methods

        private void UpdateChangeTimeStamp()
        {
            LastChangeTimeStamp = DateTime.Now;
            FirePropertyChanged("LastChangeTimeStamp");
        }

        private void FirePropertyChanged(string id)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(id));
            }
        }

        #endregion
    }
}