﻿using System;
using System.ComponentModel;
using System.Windows.Media;
using Dapfor.Wpf.Converters;

namespace Dapfor.Wpf.Grid.Demo.Examples.DataBinding.SharedData
{
    public class DataObject : INotifyPropertyChanged
    {
        #region Fields

        private readonly string _id;
        private bool _boolValue;
        private double _trackBar;
        private double _progress;
        private int _rating;
        private Color _color;

        #endregion

        public DataObject(string id, Random r)
        {
            _id = id;
            if (r != null)
            {
                _boolValue = r.Next()%2 == 1;
                _trackBar = r.Next()%100;
                _progress = r.Next() % 100;
                _rating = r.Next()%5;
                _color = Color.FromRgb((byte)r.Next(255), (byte)r.Next(255), (byte)r.Next(255));
            }
        }

        public string Id
        {
            get { return _id; }
        }

        [DoubleConverter(0)]
        public double TrackBar
        {
            get { return _trackBar; }
            set
            {
                _trackBar = value;
                NotifyUpdate("TrackBar");
            }
        }

        public double Progress
        {
            get { return _progress; }
            set
            {
                _progress = value;
                NotifyUpdate("Progress");
            }
        }

        public int Rating
        {
            get { return _rating; }
            set
            {
                _rating = value;
                NotifyUpdate("Rating");
            }
        }

        public bool BoolValue
        {
            get { return _boolValue; }
            set
            {
                _boolValue = value;
                NotifyUpdate("BoolValue");
            }
        }

        public Color Color
        {
            get { return _color; }
            set
            {
                _color = value;
                NotifyUpdate("Color");
            }
        }

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        #endregion

        private void NotifyUpdate(string fieldId)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(fieldId));
            }
        }
    }
}