﻿using System;
using System.Drawing;
using System.Globalization;
using System.Runtime.InteropServices;
using System.Windows.Data;
using System.Windows.Media.Imaging;
using Dapfor.Wpf.Controls;

namespace Dapfor.Wpf.Grid.Demo.Examples.Miscellaneous.FileExplorer
{
    public class FileToImageIconConverter : IValueConverter
    {
        public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
        {
            BitmapSource bitmap = null;
            Cell cell = (Cell) value;
            IExplorable explorable = cell != null ? (IExplorable) cell.Row.DataObject : null;
            if (explorable != null)
            {
                if(System.IO.File.Exists(explorable.FullName))
                {
                    using (Icon sysicon = Icon.ExtractAssociatedIcon(explorable.FullName))
                    {
                        if (sysicon != null)
                        {
                            bitmap = System.Windows.Interop.Imaging.CreateBitmapSourceFromHIcon(
                              sysicon.Handle,
                              System.Windows.Int32Rect.Empty,
                              BitmapSizeOptions.FromEmptyOptions());
                        }
                    }    
                }
                else if(System.IO.Directory.Exists(explorable.FullName))
                {
                    var shinfo = new SHFILEINFO();
                    IntPtr hImgSmall = SHGetFileInfo(explorable.FullName, 0, ref shinfo, Marshal.SizeOf(shinfo), SHGFI_ICON | SHGFI_SMALLICON | SHGFI_LARGEICON);
                    if (hImgSmall.ToInt32() != 0 && shinfo.hIcon.ToInt32() != 0)
                    {
                        using (Icon myIcon = Icon.FromHandle(shinfo.hIcon))
                        {
                            bitmap = System.Windows.Interop.Imaging.CreateBitmapSourceFromHIcon(
                                myIcon.Handle,
                                System.Windows.Int32Rect.Empty,
                                BitmapSizeOptions.FromEmptyOptions());    
                        }
                        DestroyIcon(shinfo.hIcon);
                    }
                }

                
            }
            return bitmap;
        }

        public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
        {
            return Binding.DoNothing;
        }

        #region PInvoke methods to extract directory's icon

        public const int SHGFI_ICON = 0x100;
        public const int SHGFI_SMALLICON = 0x1;
        public const int SHGFI_LARGEICON = 0x0;

        public struct SHFILEINFO
        {
            // Handle to the icon representing the file
            public IntPtr hIcon;

            // Index of the icon within the image list
            public int iIcon;

            // Various attributes of the file
            public uint dwAttributes;

            // Path to the file
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 256)]
            public string szDisplayName;

            // File type
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 80)]
            public string szTypeName;
        };

        [DllImport("Shell32.dll")]
        public static extern IntPtr SHGetFileInfo(string pszPath, uint dwFileAttributes, ref SHFILEINFO psfi, int cbFileInfo, uint uFlags);

        [DllImport("user32.dll", SetLastError = true)]
        [return: MarshalAs(UnmanagedType.Bool)]
        public static extern bool DestroyIcon(IntPtr hIcon);

        #endregion

    }
}
