﻿using System;
using System.Windows.Media;
using System.Windows.Threading;
using Dapfor.Wpf.Controls;
using Dapfor.Wpf.Grid.Demo.Internal;

namespace Dapfor.Wpf.Grid.Demo.Examples.Miscellaneous.Trading
{
    /// <summary>
    /// Interaction logic for RealTimeUpdatesDemo.xaml
    /// </summary>
    [Example("Miscellaneous", "Trading blotter", "Description_TradingDemo")]
    public partial class TradingDemo : IDisposable
    {
        private readonly DispatcherTimer _timer = new DispatcherTimer();
        private readonly Random _random = new Random();
        private readonly ShareCollection _shares;
        private readonly Brush _greenBrush;
        private readonly Brush _redBrush;
        private readonly Brush _updateBrush;
        
        public TradingDemo()
        {
            InitializeComponent();

            _shares = (ShareCollection)FindResource("cac40");
            _timer.Interval = TimeSpan.FromMilliseconds(1);
            _timer.Tick += OnTick;
            _timer.Start();

            _greenBrush = (Brush) FindResource("greenBrush");
            _redBrush = (Brush)FindResource("redBrush");
            _updateBrush = (Brush)FindResource("updateBrush");

            grid.Rows[5].Focused = true;
            grid.Rows[5].Selected = true;
            grid.Rows[6].Selected = true;
            grid.Rows[7].Selected = true;
            grid.Rows[8].Selected = true;
            grid.Rows[11].Selected = true;
            grid.Rows[12].Selected = true;
        }


        private void OnTick(object sender, EventArgs e)
        {
            Share data = _shares[_random.Next(_shares.Count)];
            {
                if (_random.Next() % 2 == 0)
                {
                    data.Bid = data.Last - data.Last * (1f / _random.Next(500, 1000));
                    data.BidQty = _random.Next(1, 50);

                    double delta = data.Last * (1f / _random.Next(500, 1000));
                    delta = _random.Next() % 2 == 0 ? delta : -delta;
                    data.Last = Math.Max(data.Last + delta, data.Open / 2);
                    data.LastQty = _random.Next(1, 50);
                }
                else
                {
                    data.Ask = data.Last + data.Last * (1f / _random.Next(500, 1000));
                    data.AskQty = _random.Next(1, 50);
                }

                if (_random.Next() % 10 == 0)
                {
                    //Update state
                    data = _shares[_random.Next(_shares.Count)];
                    int v = _random.Next(100);
                    if (v > 5)
                    {
                        data.State = State.Ok;
                    }
                    else data.State = v > 2 ? State.PartialOk : State.Nok;
                }
            }
        }

        public void Dispose()
        {
            _timer.Stop();
        }

        private void grid_RowUpdated(object sender, GridControlRowUpdateEventArgs e)
        {
            Share share = (Share) e.Row.DataObject;
            if (e.DataField != null && share != null)
            {
                switch (e.DataField.Id)
                {
                    case Share.FieldBid:
                        {
                            Brush color = share.Bid > share.PrevBid ? _redBrush : _greenBrush;
                            e.Row[e.DataField.Id].Blink(TimeSpan.FromMilliseconds(2000), color);
                        }
                        break;

                    case Share.FieldAsk:
                        {
                            Brush color = share.Ask > share.PrevAsk ? _redBrush : _greenBrush;
                            e.Row[e.DataField.Id].Blink(TimeSpan.FromMilliseconds(2000), color);
                        }
                        break;

                    case Share.FieldLast:
                        {
                            e.Row[e.DataField.Id].Blink(TimeSpan.FromMilliseconds(500), _updateBrush);
                        }
                        break;
                }
            }
        }
    }
}
