﻿using System;
using System.Collections.ObjectModel;
using System.Threading;
using System.Windows;
using System.Windows.Controls;
using Dapfor.Wpf.Controls;
using Dapfor.Wpf.Grid.Demo.Internal;

namespace Dapfor.Wpf.Grid.Demo.Examples.WhyUs.ThreadsafeEventHandling
{
    /// <summary>
    /// Interaction logic for ThreadsafeEventHandlingDemo.xaml
    /// </summary>
    [Example("Why us?", "Threadsafe event handling", "Description_WhyUsThreadsafeEventHandling")]
    public partial class ThreadsafeEventHandlingDemo : UserControl
    {
        static readonly Random Random = new Random();
        readonly ObservableCollection<CustomDataObject> _collection = new ObservableCollection<CustomDataObject>();

        public ThreadsafeEventHandlingDemo()
        {
            InitializeComponent();

            for (int i = 0; i < 20; ++i)
            {
                _collection.Add(new CustomDataObject());
            }

            grid.ItemsSource = _collection;

            grid.Headers[0]["Id0"].SortDirection = SortDirection.Ascending;
            grid.Headers[0]["Id3"].SortDirection = SortDirection.Descending;
        }

        private void OnAddItem(object sender, RoutedEventArgs e)
        {
            ThreadPool.QueueUserWorkItem(delegate
            {
                lock (_collection)
                {
                    _collection.Add(new CustomDataObject());    
                }
            });

        }

        private void OnRemoveItem(object sender, RoutedEventArgs e)
        {
            ThreadPool.QueueUserWorkItem(delegate
            {
                lock (_collection)
                {
                    if (_collection.Count > 0)
                    {
                        _collection.RemoveAt(Random.Next(_collection.Count));
                    }
                }
            });
        }

        private void OnUpdateItem(object sender, RoutedEventArgs e)
        {
            ThreadPool.QueueUserWorkItem(delegate
            {
                lock (_collection)
                {
                    if (_collection.Count > 0)
                    {
                        CustomDataObject o = _collection[Random.Next(_collection.Count)];
                        switch (Random.Next(7))
                        {
                            case 0: o.V0 = Random.Next(); break;
                            case 1: o.V1 = Random.Next(); break;
                            case 2: o.V2 = Random.Next(); break;
                            case 3: o.V3 = Random.Next(); break;
                            case 4: o.V4 = Random.Next(); break;
                            case 5: o.V5 = Random.Next(); break;
                            case 6: o.V6 = Random.Next(); break;
                        }
                    }
                }
            });

        }
    }
}
