﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Media;

namespace Dapfor.Wpf.Grid.Demo.GenericControls
{
    public partial class HistoryControl
    {
        public static readonly DependencyProperty QuoteCountProperty = DependencyProperty.Register("QuoteCount", typeof(int), typeof(HistoryControl), new FrameworkPropertyMetadata(40));
        public int QuoteCount
        {
            get { return (int)GetValue(QuoteCountProperty); }
            set {SetValue(QuoteCountProperty, value); }
        }
        

        public HistoryControl()
        {
            InitializeComponent();
        }

        public static readonly DependencyProperty SourceProperty = DependencyProperty.Register("Source", typeof(IList), typeof(HistoryControl), new FrameworkPropertyMetadata(null, FrameworkPropertyMetadataOptions.None, null, OnCoerceSource));
        public IList Source
        {
            get { return (IList)GetValue(SourceProperty); }
            set { SetValue(SourceProperty, value); }
        }

        private static object OnCoerceSource(DependencyObject d, object value)
        {
            var control = (HistoryControl)d;
            control.InvalidateMeasure();
            return value;
        }

        protected override Size MeasureOverride(Size constraint)
        {
            constraint = base.MeasureOverride(constraint);
            InvalidateVisual();
            return constraint;
        }

        protected override void OnRender(DrawingContext drawingContext)
        {
            var quotes = Source as IList<double>;
            if(quotes != null)
            {
                double? min = null;
                double? max = null;
                foreach (var quote in quotes)
                {
                    min = min == null ? quote : Math.Min(min.Value, quote);
                    max = max == null ? quote : Math.Max(max.Value, quote);
                }

                PathGeometry pathGeometry = new PathGeometry();
                Size size = RenderSize;
                if (min != null)
                {
                    if (quotes.Count > 1)
                    {
                        for (int i = 0; i < QuoteCount; ++i)
                        {
                            if(i < quotes.Count - 1)
                            {
                                double quote1 = quotes[i];
                                double quote2 = quotes[i + 1];
                                double x1 = size.Width - i * size.Width / (QuoteCount - 1);
                                double x2 = size.Width - (i + 1) * size.Width / (QuoteCount - 1);

                                double k1 = max.Value > min.Value ? (max.Value - quote1) / (max.Value - min.Value) : 0.5;
                                double k2 = max.Value > min.Value ? (max.Value - quote2) / (max.Value - min.Value) : 0.5;
                                double y1 = k1 * size.Height;
                                double y2 = k2 * size.Height;

                                pathGeometry.AddGeometry(new LineGeometry(new Point(x1, y1), new Point(x2, y2)));    
                            }

                            
                        }
                    }
                }

                Pen p = new Pen(Foreground, 1);
                drawingContext.DrawGeometry(Background, p, pathGeometry);
            }
        }
    }
}
