﻿using System;
using System.Collections.Generic;
using System.Windows.Controls;

namespace Dapfor.Wpf.Grid.Demo.Internal
{
    public class Examples
    {
        private static readonly List<ExampleGroup> ExampleCollection = new List<ExampleGroup>();

        public static List<ExampleGroup> ExampleGroups
        {
            get { return ExampleCollection; }
        }

        static Examples()
        {
            Initialize();
        }

        public static void Initialize()
        {
            ExampleCollection.Clear();

            foreach (Type type in typeof (App).Assembly.GetTypes())
            {
                if (type.IsSubclassOf(typeof (Control)))
                {
                    var attributes = (ExampleAttribute[]) type.GetCustomAttributes(typeof (ExampleAttribute), false);
                    if (attributes.Length > 0)
                    {
                        ExampleGroup group = GetOrCreateGroup(attributes[0].Group);
                        group.Examples.Add(new Example(attributes[0].Name, attributes[0].Description, type));
                    }
                }
            }
            ExampleCollection.Sort((x, y) => String.Compare(x.Name, y.Name, StringComparison.OrdinalIgnoreCase));
            foreach (var exampleGroup in ExampleCollection)
            {
                exampleGroup.Examples.Sort((x, y) => String.Compare(x.Name, y.Name, StringComparison.OrdinalIgnoreCase));
            }
        }

        private static ExampleGroup GetOrCreateGroup(string name)
        {
            ExampleGroup group = null;

            foreach (ExampleGroup exampleGroup in ExampleCollection)
            {
                if (Equals(exampleGroup.Name, name))
                {
                    group = exampleGroup;
                    break;
                }
            }

            if (group == null)
            {
                group = new ExampleGroup(name);
                ExampleCollection.Add(group);
            }

            return group;
        }
    }
}